"use strict";(self.webpackChunkhello_elementor=self.webpackChunkhello_elementor||[]).push([[380],{691:function(e,o,n){n.d(o,{A:function(){return t}});var l=n(1609),i=n.n(l),r=n(4623),t=i().forwardRef((e,o)=>i().createElement(r.A,{...e,ref:o}))},4380:function(e,o,n){n.r(o),n.d(o,{default:function(){return r}});var l=n(1609),i=n(691),r=l.forwardRef((e,o)=>l.createElement(i.A,{viewBox:"0 0 24 24",...e,ref:o},l.createElement("path",{fillRule:"evenodd",clipRule:"evenodd",d:"M5 4.75C4.86193 4.75 4.75 4.86193 4.75 5V7.25H19.25V5C19.25 4.86193 19.1381 4.75 19 4.75H5ZM20.75 5C20.75 4.0335 19.9665 3.25 19 3.25H5C4.0335 3.25 3.25 4.0335 3.25 5V19C3.25 19.9665 4.0335 20.75 5 20.75H19C19.9665 20.75 20.75 19.9665 20.75 19V5ZM19.25 8.75H4.75V19C4.75 19.1381 4.86193 19.25 5 19.25H19C19.1381 19.25 19.25 19.1381 19.25 19V8.75Z"})))},4623:function(e,o,n){var l=n(8168),i=n(8587),r=n(1609),t=n(4164),c=n(5659),a=n(8466),s=n(3541),u=n(1848),d=n(5099),f=n(790);const v=["children","className","color","component","fontSize","htmlColor","inheritViewBox","titleAccess","viewBox"],h=(0,u.Ay)("svg",{name:"MuiSvgIcon",slot:"Root",overridesResolver:(e,o)=>{const{ownerState:n}=e;return[o.root,"inherit"!==n.color&&o[`color${(0,a.A)(n.color)}`],o[`fontSize${(0,a.A)(n.fontSize)}`]]}})(({theme:e,ownerState:o})=>{var n,l,i,r,t,c,a,s,u,d,f,v,h;return{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:o.hasSvgAsChild?void 0:"currentColor",flexShrink:0,transition:null==(n=e.transitions)||null==(l=n.create)?void 0:l.call(n,"fill",{duration:null==(i=e.transitions)||null==(i=i.duration)?void 0:i.shorter}),fontSize:{inherit:"inherit",small:(null==(r=e.typography)||null==(t=r.pxToRem)?void 0:t.call(r,20))||"1.25rem",medium:(null==(c=e.typography)||null==(a=c.pxToRem)?void 0:a.call(c,24))||"1.5rem",large:(null==(s=e.typography)||null==(u=s.pxToRem)?void 0:u.call(s,35))||"2.1875rem"}[o.fontSize],color:null!=(d=null==(f=(e.vars||e).palette)||null==(f=f[o.color])?void 0:f.main)?d:{action:null==(v=(e.vars||e).palette)||null==(v=v.action)?void 0:v.active,disabled:null==(h=(e.vars||e).palette)||null==(h=h.action)?void 0:h.disabled,inherit:void 0}[o.color]}}),m=r.forwardRef(function(e,o){const n=(0,s.A)({props:e,name:"MuiSvgIcon"}),{children:u,className:m,color:S="inherit",component:p="svg",fontSize:A="medium",htmlColor:g,inheritViewBox:w=!1,titleAccess:z,viewBox:C="0 0 24 24"}=n,x=(0,i.A)(n,v),y=r.isValidElement(u)&&"svg"===u.type,R=(0,l.A)({},n,{color:S,component:p,fontSize:A,instanceFontSize:e.fontSize,inheritViewBox:w,viewBox:C,hasSvgAsChild:y}),V={};w||(V.viewBox=C);const B=(e=>{const{color:o,fontSize:n,classes:l}=e,i={root:["root","inherit"!==o&&`color${(0,a.A)(o)}`,`fontSize${(0,a.A)(n)}`]};return(0,c.A)(i,d.E,l)})(R);return(0,f.jsxs)(h,(0,l.A)({as:p,className:(0,t.A)(B.root,m),focusable:"false",color:g,"aria-hidden":!z||void 0,role:z?"img":void 0,ref:o},V,x,y&&u.props,{ownerState:R,children:[y?u.props.children:u,z?(0,f.jsx)("title",{children:z}):null]}))});m.muiName="SvgIcon",o.A=m},5099:function(e,o,n){n.d(o,{E:function(){return r}});var l=n(8413),i=n(3990);function r(e){return(0,i.Ay)("MuiSvgIcon",e)}(0,l.A)("MuiSvgIcon",["root","colorPrimary","colorSecondary","colorAction","colorError","colorDisabled","fontSizeInherit","fontSizeSmall","fontSizeMedium","fontSizeLarge"])}}]);