// Spectrum Colorpicker
// Turkish (tr) localization
// https://github.com/bgrins/spectrum

(function ( $ ) {

    var localization = $.spectrum.localization["tr"] = {
		cancelText: "iptal",
		chooseText: "tamam"
	};

    $.extend($.fn.spectrum.defaults, localization);

})( jQuery );
