document.addEventListener("DOMContentLoaded",()=>{document.querySelectorAll('a[href="admin.php?page=hello-elementor-ai-site-planner"], a[href="admin.php?page=hello-elementor-theme-builder"]').forEach(e=>{e.setAttribute("target","_blank")})});