"use strict";(self.webpackChunkhello_elementor=self.webpackChunkhello_elementor||[]).push([[612],{691:function(e,o,n){n.d(o,{A:function(){return t}});var l=n(1609),i=n.n(l),r=n(4623),t=i().forwardRef((e,o)=>i().createElement(r.A,{...e,ref:o}))},3612:function(e,o,n){n.r(o),n.d(o,{default:function(){return r}});var l=n(1609),i=n(691),r=l.forwardRef((e,o)=>l.createElement(i.A,{viewBox:"0 0 24 24",...e,ref:o},l.createElement("path",{fillRule:"evenodd",clipRule:"evenodd",d:"M7 3.75C6.66848 3.75 6.35054 3.8817 6.11612 4.11612C5.8817 4.35054 5.75 4.66848 5.75 5V19C5.75 19.3315 5.8817 19.6495 6.11612 19.8839C6.35054 20.1183 6.66848 20.25 7 20.25H17C17.3315 20.25 17.6495 20.1183 17.8839 19.8839C18.1183 19.6495 18.25 19.3315 18.25 19V8.75H15C14.5359 8.75 14.0908 8.56563 13.7626 8.23744C13.4344 7.90925 13.25 7.46413 13.25 7V3.75H7ZM14.75 4.81066L17.1893 7.25H15C14.9337 7.25 14.8701 7.22366 14.8232 7.17678C14.7763 7.12989 14.75 7.0663 14.75 7V4.81066ZM5.05546 3.05546C5.57118 2.53973 6.27065 2.25 7 2.25H14C14.1989 2.25 14.3897 2.32902 14.5303 2.46967L19.5303 7.46967C19.671 7.61032 19.75 7.80109 19.75 8V19C19.75 19.7293 19.4603 20.4288 18.9445 20.9445C18.4288 21.4603 17.7293 21.75 17 21.75H7C6.27065 21.75 5.57118 21.4603 5.05546 20.9445C4.53973 20.4288 4.25 19.7293 4.25 19V5C4.25 4.27065 4.53973 3.57118 5.05546 3.05546Z"})))},4623:function(e,o,n){var l=n(8168),i=n(8587),r=n(1609),t=n(4164),c=n(5659),a=n(8466),s=n(3541),u=n(1848),d=n(5099),f=n(790);const v=["children","className","color","component","fontSize","htmlColor","inheritViewBox","titleAccess","viewBox"],h=(0,u.Ay)("svg",{name:"MuiSvgIcon",slot:"Root",overridesResolver:(e,o)=>{const{ownerState:n}=e;return[o.root,"inherit"!==n.color&&o[`color${(0,a.A)(n.color)}`],o[`fontSize${(0,a.A)(n.fontSize)}`]]}})(({theme:e,ownerState:o})=>{var n,l,i,r,t,c,a,s,u,d,f,v,h;return{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:o.hasSvgAsChild?void 0:"currentColor",flexShrink:0,transition:null==(n=e.transitions)||null==(l=n.create)?void 0:l.call(n,"fill",{duration:null==(i=e.transitions)||null==(i=i.duration)?void 0:i.shorter}),fontSize:{inherit:"inherit",small:(null==(r=e.typography)||null==(t=r.pxToRem)?void 0:t.call(r,20))||"1.25rem",medium:(null==(c=e.typography)||null==(a=c.pxToRem)?void 0:a.call(c,24))||"1.5rem",large:(null==(s=e.typography)||null==(u=s.pxToRem)?void 0:u.call(s,35))||"2.1875rem"}[o.fontSize],color:null!=(d=null==(f=(e.vars||e).palette)||null==(f=f[o.color])?void 0:f.main)?d:{action:null==(v=(e.vars||e).palette)||null==(v=v.action)?void 0:v.active,disabled:null==(h=(e.vars||e).palette)||null==(h=h.action)?void 0:h.disabled,inherit:void 0}[o.color]}}),m=r.forwardRef(function(e,o){const n=(0,s.A)({props:e,name:"MuiSvgIcon"}),{children:u,className:m,color:S="inherit",component:p="svg",fontSize:C="medium",htmlColor:A,inheritViewBox:g=!1,titleAccess:w,viewBox:z="0 0 24 24"}=n,x=(0,i.A)(n,v),y=r.isValidElement(u)&&"svg"===u.type,R=(0,l.A)({},n,{color:S,component:p,fontSize:C,instanceFontSize:e.fontSize,inheritViewBox:g,viewBox:z,hasSvgAsChild:y}),V={};g||(V.viewBox=z);const B=(e=>{const{color:o,fontSize:n,classes:l}=e,i={root:["root","inherit"!==o&&`color${(0,a.A)(o)}`,`fontSize${(0,a.A)(n)}`]};return(0,c.A)(i,d.E,l)})(R);return(0,f.jsxs)(h,(0,l.A)({as:p,className:(0,t.A)(B.root,m),focusable:"false",color:A,"aria-hidden":!w||void 0,role:w?"img":void 0,ref:o},V,x,y&&u.props,{ownerState:R,children:[y?u.props.children:u,w?(0,f.jsx)("title",{children:w}):null]}))});m.muiName="SvgIcon",o.A=m},5099:function(e,o,n){n.d(o,{E:function(){return r}});var l=n(8413),i=n(3990);function r(e){return(0,i.Ay)("MuiSvgIcon",e)}(0,l.A)("MuiSvgIcon",["root","colorPrimary","colorSecondary","colorAction","colorError","colorDisabled","fontSizeInherit","fontSizeSmall","fontSizeMedium","fontSizeLarge"])}}]);