(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-790f12ce"],{"281e":function(c,t,i){"use strict";i.r(t);var o=function(){var c=this,t=c._self._c;return t("div",{staticClass:"cky-section cky-section-policies cky-zero--padding cky-zero--margin"},[t("div",{staticClass:"cky-row"},[t("div",{staticClass:"cky-col-12"},[t("cky-connect-success")],1)]),c._m(0),t("div",{staticClass:"cky-section-content"},[t("div",{staticClass:"cky-row"},[t("div",{staticClass:"cky-col-6"},[t("div",{staticClass:"cky-card cky-card-policy"},[t("div",{staticClass:"cky-card-title"},[t("h3",[c._v(" "+c._s(c.$i18n.__("Privacy Policy Generator","cookie-law-info"))+" ")])]),t("div",{staticClass:"cky-policy-description"},[t("p",[c._v(" "+c._s(c.$i18n.__("Generate a privacy policy to inform users about your website’s data collection practices.","cookie-law-info"))+" ")])]),t("div",{staticClass:"cky-policy-icon cky-align-center"},[t("cky-icon",{attrs:{icon:"privacy-policy",width:"100px",height:"145px",color:"#8893a1"}})],1),t("div",{staticClass:"cky-policy-btn"},[t("a",{staticClass:"cky-button cky-button-primary cky-button-medium cky-external-link",staticStyle:{"text-decoration":"none"},attrs:{href:"https://privacypolicy.cookieyes.com/privacy-policy-generator",target:"_blank"}},[c._v(c._s(c.$i18n.__("Generate Privacy Policy","cookie-law-info"))+" ")])])])]),t("div",{staticClass:"cky-col-6"},[t("div",{staticClass:"cky-card cky-card-policy"},[t("div",{staticClass:"cky-card-title"},[t("h3",[c._v(" "+c._s(c.$i18n.__("Cookie Policy Generator","cookie-law-info"))+" ")])]),t("div",{staticClass:"cky-policy-description"},[t("p",[c._v(" "+c._s(c.$i18n.__("Generate a cookie policy by connecting to a free CookieYes account and inform users about your site's use of cookies.","cookie-law-info"))+" ")])]),t("div",{staticClass:"cky-policy-icon cky-align-center"},[t("cky-icon",{attrs:{icon:"cookie-policy",width:"100px",height:"145px",color:"#8893a1"}})],1),c.connected?t("div",{staticClass:"cky-policy-btn"},[t("a",{staticClass:"cky-button cky-button-primary cky-button-medium",staticStyle:{"text-decoration":"none"},attrs:{href:"https://cookiepolicy.cookieyes.com/cookie-policy-generator",target:"_blank"}},[c._v(c._s(c.$i18n.__("Generate Cookie Policy","cookie-law-info"))+" ")])]):t("div",{staticClass:"cky-policy-btn"},[t("cky-button",{ref:"ckyButtonConnect",staticClass:"cky-button-medium",nativeOn:{click:function(t){return c.connectToApp()}}},[c._v(" "+c._s(c.$i18n.__("New? Create a Free Account","cookie-law-info"))+" "),t("template",{slot:"loader"},[c._v(c._s(c.$i18n.__("Connecting to cookieyes.com...","cookie-law-info")))])],2),t("p",{staticStyle:{"margin-top":"10px",position:"absolute",left:"0",right:"0"}},[c._v(" "+c._s(c.$i18n.__("Already have an account?","cookie-law-info"))+" "),t("a",{attrs:{href:""},on:{click:function(t){return t.preventDefault(),c.connectToApp(!0)}}},[c._v(c._s(c.$i18n.__("Connect your existing account","cookie-law-info")))])])],1)])])])])])},e=[function(){var c=this,t=c._self._c;return t("div",{staticClass:"cky-section-header cky-align-center cky-justify-between"},[t("div",{staticClass:"cky-section-header-actions cky-align-center"})])}],n=i("1f3d"),a=i("c068"),s=i("2f62"),r=i("919d"),l={name:"CkyPolicies",mixins:[a["a"]],components:{CkyIcon:n["a"],CkyConnectSuccess:r["a"]},data(){return{}},methods:{},computed:{...Object(s["d"])("settings",["options"]),connected(){return this.options.account.connected&&this.options.account.connected||!1},currentTabComponent:function(){return"tab-"+this.currentTab.toLowerCase()}}},y=l,k=(i("cc6e"),i("2877")),p=Object(k["a"])(y,o,e,!1,null,null,null);t["default"]=p.exports},"64f6":function(c,t,i){},cc6e:function(c,t,i){"use strict";i("64f6")}}]);