"use strict";(self.webpackChunkhello_elementor=self.webpackChunkhello_elementor||[]).push([[835],{691:function(o,e,n){n.d(e,{A:function(){return t}});var l=n(1609),i=n.n(l),r=n(4623),t=i().forwardRef((o,e)=>i().createElement(r.A,{...o,ref:e}))},1835:function(o,e,n){n.r(e);var l=n(1609),i=n(691),r=n(790);const t=l.forwardRef((o,e)=>(0,r.jsx)(i.A,{viewBox:"0 0 24 24",...o,ref:e,children:(0,r.jsx)("path",{fillRule:"evenodd",clipRule:"evenodd",d:"M3.95964 2.64459C2.76302 2.64459 1.79297 3.61464 1.79297 4.81126V12.8113C1.79297 14.0079 2.76302 14.9779 3.95964 14.9779H16.6263C17.8229 14.9779 18.793 14.0079 18.793 12.8113V4.81126C18.793 3.61464 17.8229 2.64459 16.6263 2.64459H3.95964ZM0.79297 4.81126C0.79297 3.06236 2.21073 1.64459 3.95964 1.64459H16.6263C18.3752 1.64459 19.793 3.06236 19.793 4.81126V12.8113C19.793 14.5602 18.3752 15.9779 16.6263 15.9779H3.95964C2.21073 15.9779 0.79297 14.5602 0.79297 12.8113V4.81126ZM7.71329 4.87616C7.87004 4.78741 8.06242 4.78983 8.21688 4.88251L13.5502 7.88251C13.7008 7.97287 13.793 8.13563 13.793 8.31126C13.793 8.48689 13.7008 8.64964 13.5502 8.74001L8.21688 11.74C8.06242 11.8327 7.87004 11.8351 7.71329 11.7464C7.55653 11.6576 7.45964 11.4914 7.45964 11.3113V5.31126C7.45964 5.13112 7.55653 4.96491 7.71329 4.87616ZM8.45964 6.19435V10.4282L12.3211 8.31126L8.45964 6.19435Z"})}));e.default=t},4623:function(o,e,n){var l=n(8168),i=n(8587),r=n(1609),t=n(4164),c=n(5659),a=n(8466),s=n(3541),u=n(1848),d=n(5099),f=n(790);const v=["children","className","color","component","fontSize","htmlColor","inheritViewBox","titleAccess","viewBox"],h=(0,u.Ay)("svg",{name:"MuiSvgIcon",slot:"Root",overridesResolver:(o,e)=>{const{ownerState:n}=o;return[e.root,"inherit"!==n.color&&e[`color${(0,a.A)(n.color)}`],e[`fontSize${(0,a.A)(n.fontSize)}`]]}})(({theme:o,ownerState:e})=>{var n,l,i,r,t,c,a,s,u,d,f,v,h;return{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:e.hasSvgAsChild?void 0:"currentColor",flexShrink:0,transition:null==(n=o.transitions)||null==(l=n.create)?void 0:l.call(n,"fill",{duration:null==(i=o.transitions)||null==(i=i.duration)?void 0:i.shorter}),fontSize:{inherit:"inherit",small:(null==(r=o.typography)||null==(t=r.pxToRem)?void 0:t.call(r,20))||"1.25rem",medium:(null==(c=o.typography)||null==(a=c.pxToRem)?void 0:a.call(c,24))||"1.5rem",large:(null==(s=o.typography)||null==(u=s.pxToRem)?void 0:u.call(s,35))||"2.1875rem"}[e.fontSize],color:null!=(d=null==(f=(o.vars||o).palette)||null==(f=f[e.color])?void 0:f.main)?d:{action:null==(v=(o.vars||o).palette)||null==(v=v.action)?void 0:v.active,disabled:null==(h=(o.vars||o).palette)||null==(h=h.action)?void 0:h.disabled,inherit:void 0}[e.color]}}),m=r.forwardRef(function(o,e){const n=(0,s.A)({props:o,name:"MuiSvgIcon"}),{children:u,className:m,color:S="inherit",component:p="svg",fontSize:A="medium",htmlColor:C,inheritViewBox:g=!1,titleAccess:w,viewBox:x="0 0 24 24"}=n,z=(0,i.A)(n,v),y=r.isValidElement(u)&&"svg"===u.type,R=(0,l.A)({},n,{color:S,component:p,fontSize:A,instanceFontSize:o.fontSize,inheritViewBox:g,viewBox:x,hasSvgAsChild:y}),V={};g||(V.viewBox=x);const M=(o=>{const{color:e,fontSize:n,classes:l}=o,i={root:["root","inherit"!==e&&`color${(0,a.A)(e)}`,`fontSize${(0,a.A)(n)}`]};return(0,c.A)(i,d.E,l)})(R);return(0,f.jsxs)(h,(0,l.A)({as:p,className:(0,t.A)(M.root,m),focusable:"false",color:C,"aria-hidden":!w||void 0,role:w?"img":void 0,ref:e},V,z,y&&u.props,{ownerState:R,children:[y?u.props.children:u,w?(0,f.jsx)("title",{children:w}):null]}))});m.muiName="SvgIcon",e.A=m},5099:function(o,e,n){n.d(e,{E:function(){return r}});var l=n(8413),i=n(3990);function r(o){return(0,i.Ay)("MuiSvgIcon",o)}(0,l.A)("MuiSvgIcon",["root","colorPrimary","colorSecondary","colorAction","colorError","colorDisabled","fontSizeInherit","fontSizeSmall","fontSizeMedium","fontSizeLarge"])}}]);