// Spectrum Colorpicker
// Arabic (ar) localization
// https://github.com/bgrins/spectrum
(function ( $ ) {

var localization = $.spectrum.localization["ar"] = {
    cancelText: "إلغاء",
    chooseText: "إختار",
    clearText: "إرجاع الألوان على ما كانت",
    noColorSelectedText: "لم تختار أي لون",
    togglePaletteMoreText: "أكثر",
    togglePaletteLessText: "أقل"
};

$.extend($.fn.spectrum.defaults, localization);

})( jQuery );
