"use strict";(self.webpackChunkhello_elementor=self.webpackChunkhello_elementor||[]).push([[516],{691:function(o,e,n){n.d(e,{A:function(){return t}});var l=n(1609),r=n.n(l),i=n(4623),t=r().forwardRef((o,e)=>r().createElement(i.A,{...o,ref:e}))},1516:function(o,e,n){n.r(e),n.d(e,{default:function(){return i}});var l=n(1609),r=n(691),i=l.forwardRef((o,e)=>l.createElement(r.A,{viewBox:"0 0 24 24",...o,ref:e},l.createElement("path",{d:"M4.25 5C4.25 4.58579 4.58579 4.25 5 4.25H19C19.4142 4.25 19.75 4.58579 19.75 5V7C19.75 7.41421 19.4142 7.75 19 7.75C18.5858 7.75 18.25 7.41421 18.25 7V5.75H12.75V18.25H14C14.4142 18.25 14.75 18.5858 14.75 19C14.75 19.4142 14.4142 19.75 14 19.75H10C9.58579 19.75 9.25 19.4142 9.25 19C9.25 18.5858 9.58579 18.25 10 18.25H11.25V5.75H5.75V7C5.75 7.41421 5.41421 7.75 5 7.75C4.58579 7.75 4.25 7.41421 4.25 7V5Z"})))},4623:function(o,e,n){var l=n(8168),r=n(8587),i=n(1609),t=n(4164),c=n(5659),a=n(8466),s=n(3541),u=n(1848),d=n(5099),f=n(790);const v=["children","className","color","component","fontSize","htmlColor","inheritViewBox","titleAccess","viewBox"],h=(0,u.Ay)("svg",{name:"MuiSvgIcon",slot:"Root",overridesResolver:(o,e)=>{const{ownerState:n}=o;return[e.root,"inherit"!==n.color&&e[`color${(0,a.A)(n.color)}`],e[`fontSize${(0,a.A)(n.fontSize)}`]]}})(({theme:o,ownerState:e})=>{var n,l,r,i,t,c,a,s,u,d,f,v,h;return{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:e.hasSvgAsChild?void 0:"currentColor",flexShrink:0,transition:null==(n=o.transitions)||null==(l=n.create)?void 0:l.call(n,"fill",{duration:null==(r=o.transitions)||null==(r=r.duration)?void 0:r.shorter}),fontSize:{inherit:"inherit",small:(null==(i=o.typography)||null==(t=i.pxToRem)?void 0:t.call(i,20))||"1.25rem",medium:(null==(c=o.typography)||null==(a=c.pxToRem)?void 0:a.call(c,24))||"1.5rem",large:(null==(s=o.typography)||null==(u=s.pxToRem)?void 0:u.call(s,35))||"2.1875rem"}[e.fontSize],color:null!=(d=null==(f=(o.vars||o).palette)||null==(f=f[e.color])?void 0:f.main)?d:{action:null==(v=(o.vars||o).palette)||null==(v=v.action)?void 0:v.active,disabled:null==(h=(o.vars||o).palette)||null==(h=h.action)?void 0:h.disabled,inherit:void 0}[e.color]}}),m=i.forwardRef(function(o,e){const n=(0,s.A)({props:o,name:"MuiSvgIcon"}),{children:u,className:m,color:S="inherit",component:p="svg",fontSize:A="medium",htmlColor:g,inheritViewBox:w=!1,titleAccess:C,viewBox:z="0 0 24 24"}=n,x=(0,r.A)(n,v),y=i.isValidElement(u)&&"svg"===u.type,V=(0,l.A)({},n,{color:S,component:p,fontSize:A,instanceFontSize:o.fontSize,inheritViewBox:w,viewBox:z,hasSvgAsChild:y}),B={};w||(B.viewBox=z);const R=(o=>{const{color:e,fontSize:n,classes:l}=o,r={root:["root","inherit"!==e&&`color${(0,a.A)(e)}`,`fontSize${(0,a.A)(n)}`]};return(0,c.A)(r,d.E,l)})(V);return(0,f.jsxs)(h,(0,l.A)({as:p,className:(0,t.A)(R.root,m),focusable:"false",color:g,"aria-hidden":!C||void 0,role:C?"img":void 0,ref:e},B,x,y&&u.props,{ownerState:V,children:[y?u.props.children:u,C?(0,f.jsx)("title",{children:C}):null]}))});m.muiName="SvgIcon",e.A=m},5099:function(o,e,n){n.d(e,{E:function(){return i}});var l=n(8413),r=n(3990);function i(o){return(0,r.Ay)("MuiSvgIcon",o)}(0,l.A)("MuiSvgIcon",["root","colorPrimary","colorSecondary","colorAction","colorError","colorDisabled","fontSizeInherit","fontSizeSmall","fontSizeMedium","fontSizeLarge"])}}]);