!function(){"use strict";var n={n:function(t){var a=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(a,{a}),a},d:function(t,a){for(var e in a)n.o(a,e)&&!n.o(t,e)&&Object.defineProperty(t,e,{enumerable:!0,get:a[e]})},o:function(n,t){return Object.prototype.hasOwnProperty.call(n,t)}};const t=jQuery;var a;(a=n.n(t)())(function(){window.rankMathFront={init:function(){this.adminMenu()},adminMenu:function(){var n=a("#wp-admin-bar-rank-math-mark-me"),t=this;n.on("click",".mark-page-as a",function(n){n.preventDefault(),t.ajax("mark_page_as",{objectID:rankMath.objectID,objectType:rankMath.objectType,what:a(this).attr("href").replace("#","")}),a(this).find(".dashicons").length?a(this).find(".dashicons").remove():a(this).prepend('<span class="dashicons dashicons-yes" style="font-family:dashicons;font-size:19px;line-height:28px;margin:0 3px 0 -4px;"></span>')})},ajax:function(n,t,e){return a.ajax({url:rankMath.ajaxurl,type:e||"POST",dataType:"json",data:a.extend(!0,{action:"rank_math_"+n,security:rankMath.security},t)})}},window.rankMathFront.init()})}();