// Spectrum Colorpicker
// Italian (it) localization
// https://github.com/bgrins/spectrum

(function ( $ ) {

    var localization = $.spectrum.localization["it"] = {
		cancelText: "annulla",
		chooseText: "scegli",
		clearText: "Annulla selezione colore",
		noColorSelectedText: "Nessun colore selezionato"
	};

    $.extend($.fn.spectrum.defaults, localization);

})( jQuery );
